// =========================================================================================================
//  Bullet Tracker - (part of the Mobile Target Range module)
//  Author: Kronzky (www.kronzky.info / kronzky@gmail.com)
// =========================================================================================================

#include "functions.sqf";

// only track one bullet at a time
KRON_tracking=true;
clearRadio;

// temporarily turn off global ballistics variable, to suppress the elevation hint
_showBallistics = KRON_Ballistics;
KRON_Ballistics = false;

_debug = 1; //1= write to debug log, 2=echo to debugView (dev. version only)
_camOffset = [-2,.5]; // offset for bullet cam [behind, above]

_ammo = _this select 4;
_bullet = nearestObject [player, _ammo];

KRON_BallisticsChart=[];
titletext["","plain"];

_dirshot = [] call KRON_weapondir;

setAccTime 1;
if (isNull _bullet) exitWith {KRON_HQ globalchat format["Could not track bullet type %1",_ammo]};
KRON_HQ globalchat format["tracking %1",typeOf _bullet];

_zASL = (getPosASL _bullet) select 2;
_zPos = (getPos _bullet) select 2;
_zOff = _zASL;
_wdir = direction player;
//KRON_BallisticsChart set [0,[0,0,[_zASL-_zOff,2] call _cutDecimals,0]];
if (_showBallistics) then {
	setAccTime .5;
} else {
	setAccTime 1;
};
_starttime=time;

_dist=0;
_di=1; // distance index (entry line in listbox)
_lastpos=[0,0,0];
_lastZ=999;
_path="init";
KRON_AbortCam = false;
if ((KRON_BulletCam>0) && (isNil("KRON_cam"))) then {
	// cannot have ballistics chart AND camera
	KRON_Ballistics = false;
	_showBallistics = false;
	showCinemaBorder false;
	KRON_cam = "camera" camcreate [0,0,0];
	KRON_cam cameraeffect ["internal", "back"];
	setAccTime KRON_BulletCam;
	KRON_cam camSetFocus [2,0];
	
	waitUntil {
		if !(isNull _bullet) then {
			_dist=_bullet distance player; 
			_lastpos=getPos _bullet;
			_bulletdir = getDir _bullet;
			_sindir = sin _bulletdir;
			_cosdir = cos _bulletdir;
			KRON_cam camsetpos [(_lastpos select 0)+_sindir*(_camOffset select 0), (_lastpos select 1)+_cosdir*(_camOffset select 0), ((_lastpos select 2)+(_camOffset select 1)) max .5]; 
			KRON_cam camsettarget [(_lastpos select 0)+_sindir*100, (_lastpos select 1)+_cosdir*100, ((_lastpos select 2)+.1) max .25]; 
			KRON_cam camcommit 0;
			if ((time-_starttime>15*KRON_BulletCam) && (accTime<1)) then {KRON_HQ sidechat "slow-mo override"; setAccTime 1};
			sleep .0001;
		}; 
		((isNull _bullet) || (KRON_AbortCam))
	};
	setAccTime 1;
	if !(KRON_AbortCam) then {
		sleep 1;
	};
	KRON_cam cameraeffect ["terminate", "back"]; 
	camdestroy KRON_cam;
	KRON_cam=nil;
	hint "";

} else {
	
	_lasttime = 0;
	while {!(isNull _bullet)} do {
		_newpos=getPosASL _bullet;
		if ((_newpos select 0)!=0) then {
			_dist=_bullet distance player; 
			_lastpos=_newpos;
			_bz=(_lastpos select 2)-_zOff;
			_di1=floor(_dist/50)+1;
			_airtime = time-_starttime;
			if ((_di1>_di) || (time-_lasttime>1)) then {
				hint format["time: %1, dist: %2, height: %3",round _airtime,round _dist,round _bz];
				_lasttime = time;
				_s=speed _bullet;
				// bullet is dropping
				if (_airtime>.5) then {
					if (_bz<=_lastZ) then {
						_path = "drop";
					} else {
						if (_path!="drop") then {
							// initial rise
							_path = "rise";
						} else {
							// if the bullet rises again, it probably bounced off somewhere, so abort the tracking
							_path = "bounce";
						};
					};
				};
				_lastZ = _bz;
				if ((_path=="bounce") || (_dist>10000)) exitWith {
					_bullet = objNull;
				};
				KRON_BallisticsChart=KRON_BallisticsChart + [[floor(_dist/50)*50,_airtime,_bz,_s]];
				_di=_di1;
			};
		};
		sleep .0001;
	};
};
player setdamage 0;

_speed=if ((time-_starttime)>0) then {round(_dist/(time-_starttime))} else {0};
_speed=if (_speed>10000) then {"n/a"} else {format["%1 m/s",_speed]};
if (KRON_BulletCam>0) then {_speed=0};
_bpos=[getPos player, _wdir, 0, _dist] call KRON_rotpoint;
_xoff=[_lastpos distance [_bpos select 0,_bpos select 1,_lastpos select 2],2] call _cutDecimals;
_xoff=if (_xoff>10) then {"n/a"} else {str _xoff};
_diststr=str _dist;
if (_dist>100000) then {
	_diststr = "n/a";
	_path = "bounce";
};

hint "";
clearRadio;

if !(_showBallistics) then {
	KRON_BallisticsChart=nil;
	_stats = format["elev:%1, dist:%2, height:%3, x-off:%4, speed:%5",_dirshot, _diststr,[(_lastpos select 2)-_zOff,2] call _cutDecimals,_xoff,_speed]; 
	if !(KRON_AbortCam) then {
		titletext [format["[%1] Impact: %2",KRON_weapon,_stats],"plain down"]; 
	};
} else {
	_di=floor(_dist/50)+1;
	_d=round(_dist);
	_lastdist=(KRON_BallisticsChart select ((count KRON_BallisticsChart)-1)) select 0;
	if ((_d!=_lastdist) && (_path!="bounce")) then {
		_z=[(_lastpos select 2)-_zOff,2] call _cutDecimals;
		KRON_BallisticsChart=KRON_BallisticsChart + [[_d,time-_starttime,_z,0]];
	};
	_dlg=createDialog "Dlg1";
	if (!_dlg) exitWith {hint "Could not open dialog.\nIs description.ext present in current folder?"}; 
	lbClear DLG1_BLLLIST;
	lbAdd [DLG1_BLLLIST,format["Weapon: %1",KRON_weapon]];
	_paste=format["Weapon: %1<br>",getText (configFile >> "cfgWeapons" >> KRON_weapon >> "displayName")];
	lbAdd [DLG1_BLLLIST,format["%1",_ammo]];
	_paste=_paste+format["Ammo: %1<br>",_ammo];
	lbAdd [DLG1_BLLLIST,format["Elevation: %1",_dirshot]];
	_paste=_paste+format["Elevation: %1<br>",_dirshot];
	lbAdd [DLG1_BLLLIST,format["X-offset: %1",_xoff]];
	_paste=_paste+format["X-offset: %1<br>",_xoff];
	lbAdd [DLG1_BLLLIST,format["Avg. speed: %1",_speed]];
	_paste=_paste+format["Avg. speed: %1<br>",_speed];
	lbAdd [DLG1_BLLLIST,format["Distance: %1 m",round _dist]];
	_paste=_paste+format["Distance: %1<br>",round _dist];
	switch (_debug) do {
		case 1: {
			debugLog format["[Ballistics] Distance,Time,Elevation,Speed,,%1",_paste];
			{echo format["[Ballistics] %1,%2,%3,%4",_x select 0,_x select 1,_x select 2,_x select 3]}forEach KRON_BallisticsChart;
		};
		case 2: {
			echo format["[Ballistics] Distance,Time,Elevation,Speed,,%1",_paste];
			{echo format["[Ballistics] %1,%2,%3,%4",_x select 0,_x select 1,_x select 2,_x select 3]}forEach KRON_BallisticsChart;
		};
	};
	lbAdd [DLG1_BLLLIST,format["[distance, time, height]:",_x select 0,_val]]; 
	_paste = _paste + format["Distance, Time, Height:<br>",_x select 0,_val]; 
	{
		_val = [_x select 2,2] call _cutDecimals;
		lbAdd [DLG1_BLLLIST,format["%1, %2, %3",_x select 0,[_x select 1,1] call fn_vbs_cutDecimals,_val]]; 
		_paste = _paste + format["%1,%2,%3<br>",_x select 0,[_x select 1,1] call fn_vbs_cutDecimals,_val];
	}forEach KRON_BallisticsChart;
	ctrlSetText [DLG1_BLLPASTE, format["%1",_paste]];	
};

setAccTime KRON_SlowMo;

if (KRON_Marker) then {
	[_lastpos] call KRON_makemarker;
};

KRON_tracking=false;
KRON_Ballistics=_showBallistics;
